/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg.grpc;

import io.grpc.Server;
import io.grpc.ServerBuilder;
import io.grpc.stub.StreamObserver;
import java.io.IOException;
import java.util.Random;

/**
 *
 * @author omozegieaziegbe
 */
public class UserServiceImpl extends UserServiceGrpc.UserServiceImplBase {

    private final Random random = new Random();

    @Override
    public void getUser(GetUserRequest request, StreamObserver<GetUserResponse> responseObserver) {

        // Simulate a 90% chance of transient failure
        if (random.nextInt(100) < 90) {
            responseObserver.onError(io.grpc.Status.UNAVAILABLE
                    .withDescription("Service unavailable")
                    .asRuntimeException());
            System.out.println("Service temporarily unavailable; will retry if the policy allows.");
        } else {
            GetUserResponse response = GetUserResponse.newBuilder()
                    .setUserId(request.getUserId())
                    .setName("Allan Gee")
                    .setEmail("allan.geee@jcg.com")
                    .build();
            responseObserver.onNext(response);
            responseObserver.onCompleted();
        }
    }

    public static void main(String[] args) throws IOException, InterruptedException {
        Server server = ServerBuilder.forPort(50051)
                .addService(new UserServiceImpl())
                .build()
                .start();

        System.out.println("Server started on port 50051");
        server.awaitTermination();
    }
}
